import { Signal } from 'solid-js';
import PouchDB from 'pouchdb';
import indexeddb from 'pouchdb-adapter-indexeddb';

import { getSettings, putSettings } from '../db/settings';
import { sleep } from '../lib/async-wait';
import {
  getAccounts,
  initialAccount,
  putAccount,
  getAccountById,
} from '../db/account';

PouchDB.plugin(indexeddb);

declare global {
  var localDb: any;
  var db: any;
}

const initDb = async () => {
  if (globalThis.localDb === undefined) {
    globalThis.localDb = new PouchDB('_local_dyomedea_', {
      adapter: 'indexeddb',
      auto_compaction: true,
    });
  }

  const accounts = await getAccounts();
  console.log({ caller: 'initDb', accounts });

  if (accounts.length === 0) {
    accounts[0] = initialAccount;
    await putAccount({ id: initialAccount.id, account: initialAccount });
  }

  const settings = await getSettings();
  let currentAccount = getAccountById(accounts, settings.currentAccountId);
  if (currentAccount === undefined) {
    settings.currentAccountId = accounts[0].id;
    await putSettings({ settings });
    currentAccount = accounts[0];
  }

  console.log({ caller: 'initDb', settings, currentAccount });
  if (globalThis.db === undefined) {
    globalThis.db = new PouchDB(currentAccount.localDb, {
      adapter: 'indexeddb',
      auto_compaction: true,
    });
  }
};

export const watchDb = async (p: { signal: Signal<any>; params: any }) => {
  const { signal } = p;
  const [, setStatus] = signal;
  console.log({ caller: 'watchDb', p });

  await initDb();
  db = globalThis.db;
  localDb = globalThis.localDb;

  const dbinfo = await db.info();
  const localDbInfo = await localDb.info();
  const tasks = PouchDB.activeTasks.list();

  setStatus({ opened: true, dbinfo, localDbinfo: localDbInfo, tasks });

  return;

  const timerId = setInterval(async () => {
    const dbinfo = await db.info();
    const tasks = PouchDB.activeTasks.list();
    if (tasks.length > 0) {
      setStatus({ status: 'ongoing', db: 'dyomedea', dbinfo, tasks });
    } else {
      setStatus({ status: 'terminated', db: 'dyomedea', dbinfo, tasks });
      clearInterval(timerId);
    }
  }, 1000);

  // db.compact();

  const openIDb = (name: string) =>
    new Promise((resolve, reject) => {
      const iDb = indexedDB.open(name);
      iDb.onerror = (event: any) => {
        console.error({
          caller: 'watchDb',
          message: 'open db error',
          target: event.target,
        });
        reject(event.target.errorCode);
      };

      iDb.onsuccess = (event: any) => {
        console.log({
          caller: 'watchDb',
          message: 'open db',
          target: event.target,
        });
        resolve(event.target.result);
      };
    });

  const getAll = (store: any) =>
    new Promise((resolve, reject) => {
      const request = store.getAll();
      request.onerror = (event: any) => {
        console.error({
          caller: 'watchDb',
          message: 'getAll error',
          target: event.target,
        });
        reject(event.target.errorCode);
      };

      request.onsuccess = (event: any) => {
        console.log({
          caller: 'watchDb',
          message: 'getAll',
          target: event.target,
        });
        resolve(event.target.result as any[]);
      };
    });

  const iDb = (await openIDb('_pouch__dyomedea_')) as IDBDatabase;

  const bySequence = iDb.transaction('docs', 'readonly');
  const store = bySequence.objectStore('docs');
  console.log({
    caller: 'watchDb',
    message: 'transaction opened',
    bySequence,
    store,
  });

  const docs = (await getAll(store)) as any[];
  //   setStatus({
  //     status: 'docs',
  //     db: 'dyomedea',
  //     docs,
  //   });

  docs.forEach(async (doc) => {
    const { id, rev, deleted } = doc;
    if (deleted !== 0) {
      //     const purge = await db.purge(id, rev);
      console.log({
        caller: 'watchDb',
        message: 'purging',
        id,
        rev,
        doc,
        //       purge,
      });
      await sleep(1000);
    }
  });
};
